/*
 * Copyright 2004 Digi International (www.digi.com)
 *      Scott H Kilau <Scott_Kilau at digi dot com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	NOTE: THIS IS A SHARED HEADER. DO NOT CHANGE CODING STYLE!!!
 *
 *************************************************************************
 *
 * This file is intended to contain all the kernel "differences" between the
 * various kernels that we support.
 *
 *************************************************************************/

#ifndef __DGAP_KCOMPAT_H
#define __DGAP_KCOMPAT_H

# ifndef KERNEL_VERSION
#  define KERNEL_VERSION(a,b,c)  (((a) << 16) + ((b) << 8) + (c))
# endif


#if !defined(DEFINE_SPINLOCK)
# define DEFINE_SPINLOCK(x) spinlock_t x = SPIN_LOCK_UNLOCKED  
#endif

/*
 * The kernel people changed the "shared irq" define in 2.6.18+.
 */
#ifndef IRQF_SHARED
# define IRQF_SHARED SA_SHIRQ
#endif



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

/* Sparse stuff */
# ifndef __user
#  define __user
#  define __kernel
#  define __safe
#  define __force
#  define __chk_user_ptr(x) (void)0
# endif

# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,9)

#  define PARM_STR(VAR, INIT, PERM, DESC) \
		static char *VAR = INIT; \
		char *dgap_##VAR; \
		module_param(VAR, charp, PERM); \
		MODULE_PARM_DESC(VAR, DESC);

#  define PARM_INT(VAR, INIT, PERM, DESC) \
		static int VAR = INIT; \
		int dgap_##VAR; \
		module_param(VAR, int, PERM); \
		MODULE_PARM_DESC(VAR, DESC);

#  define PARM_ULONG(VAR, INIT, PERM, DESC) \
		static ulong VAR = INIT; \
		ulong dgap_##VAR; \
		module_param(VAR, long, PERM); \
		MODULE_PARM_DESC(VAR, DESC);

# else

#  define PARM_STR(VAR, INIT, PERM, DESC) \
		static char *VAR = INIT; \
		char *dgap_##VAR; \
		MODULE_PARM(VAR, "s"); \
		MODULE_PARM_DESC(VAR, DESC);

#  define PARM_INT(VAR, INIT, PERM, DESC) \
		static int VAR = INIT; \
		int dgap_##VAR; \
		MODULE_PARM(VAR, "i"); \
		MODULE_PARM_DESC(VAR, DESC);

#  define PARM_ULONG(VAR, INIT, PERM, DESC) \
		static ulong VAR = INIT; \
		ulong dgap_##VAR; \
		MODULE_PARM(VAR, "l"); \
		MODULE_PARM_DESC(VAR, DESC);

# endif

# define DGAP_MAJOR(x)			(imajor(x))
# define DGAP_MINOR(x)			(iminor(x))
# define DGAP_TTY_MAJOR(tty)		(MAJOR(tty_devnum(tty)))
# define DGAP_TTY_MINOR(tty)		(MINOR(tty_devnum(tty)))

# define DGAP_MOD_INC_USE_COUNT(rtn)	(rtn = 1)
# define DGAP_MOD_DEC_USE_COUNT		do {} while (0)

# define DGAP_IRQRETURN_TYPE		irqreturn_t
# define DGAP_IRQ_RETURN(x)		return x;

# define PARM_GEN_IP			(PDE(file->f_dentry->d_inode))

# define DGAP_NUM_CPUS			(num_online_cpus())

# define DGAP_GET_TTY_COUNT(x)		(x->count)

/* Linux 2.6.9+ all have the new Alan Cox tty locking in it... */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,9)
#  define NEW_TTY_LOCKING
# endif

# ifndef NEW_TTY_LOCKING

#  define tty_ldisc_flush(tty) \
	if ((tty)->ldisc.flush_buffer) { \
		(tty)->ldisc.flush_buffer((tty)); \
	}

#  define tty_wakeup(tty) \
	if ((test_bit(TTY_DO_WRITE_WAKEUP, &(tty)->flags)) && \
	   (tty)->ldisc.write_wakeup) { \
		((tty)->ldisc.write_wakeup)((tty)); \
	}

#  define tty_ldisc_ref(tty) (&(tty)->ldisc)

#  define tty_ldisc_deref(ld) do {} while (0)

# endif

/* Linux 2.6.16+ all have the new Alan Cox tty buffer changes in it... */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
#  define NEW_TTY_BUFFERING
# endif

/*
 * Include support for SYSFS on all 2.6.20+ kernels.
 */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#  define DGAP_SYSFS_SUPPORT
# else
#  undef DGAP_SYSFS_SUPPORT
# endif

/*
 * They really started reworking the naming convention of
 * the MUTEX_INIT define, switching to one thing, and then
 * another.
 * This mess of ifdefs catalog the changes, while working
 * around their naming mess
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
# ifndef __SEMAPHORE_INITIALIZER
#  define DGAP_MUTEX_INITIALIZER(name)  __MUTEX_INITIALIZER(name)
# else
#  define DGAP_MUTEX_INITIALIZER(name)  __SEMAPHORE_INITIALIZER(name, 1)
# endif
#else
# define DGAP_MUTEX_INITIALIZER(name)   __MUTEX_INITIALIZER(name)
#endif

/*
 * The pci_module_init_driver() call is being depricated.
 * Switch to the new call now...
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
# define DGAP_PCI_REGISTER_DRIVER(driver) pci_register_driver(driver)
#else
# define DGAP_PCI_REGISTER_DRIVER(driver) pci_module_init(driver)
#endif

#if !defined(TTY_FLIPBUF_SIZE)
# define TTY_FLIPBUF_SIZE 512
#endif



# elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)





/* No SYSFS stuff in 2.4 */
#undef DGAP_SYSFS_SUPPORT

/* These functions ONLY exist in 2.6+ */
#define tty_register_device(a, b, c) do {} while (0)
#define tty_unregister_device(a, b) do {} while (0)

/* No Sparse stuff in 2.4.x */
# define __user
# define __kernel
# define __safe
# define __force
# define __chk_user_ptr(x) (void)0

# define PARM_STR(VAR, INIT, PERM, DESC) \
		static char *VAR = INIT; \
		char *dgap_##VAR; \
		MODULE_PARM(VAR, "s"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_INT(VAR, INIT, PERM, DESC) \
		static int VAR = INIT; \
		int dgap_##VAR; \
		MODULE_PARM(VAR, "i"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_ULONG(VAR, INIT, PERM, DESC) \
		static ulong VAR = INIT; \
		ulong dgap_##VAR; \
		MODULE_PARM(VAR, "l"); \
		MODULE_PARM_DESC(VAR, DESC);

# define DGAP_MAJOR(x)			(MAJOR((x)->i_rdev))
# define DGAP_MINOR(x)			(MINOR((x)->i_rdev))
# define DGAP_TTY_MAJOR(tty)		(MAJOR(tty->device))
# define DGAP_TTY_MINOR(tty)		(MINOR(tty->device))

# define DGAP_MOD_INC_USE_COUNT(rtn) \
	{ \
		MOD_INC_USE_COUNT; \
		rtn = 1; \
	}

# define DGAP_MOD_DEC_USE_COUNT 	MOD_DEC_USE_COUNT

# define DGAP_IRQRETURN_TYPE		void
# define DGAP_IRQ_RETURN(x)		return;

# define PARM_GEN_IP			(file->f_dentry->d_inode->u.generic_ip)

# define DGAP_NUM_CPUS			(smp_num_cpus)

/*
 * Contrary to what I have been led to believe by some of the
 * kernel guru's, not all 2.4.x kernels have min/max macro's
 * in the kernel.h header.
 * I call your attention to the Red Hat AS 2.4.9-e.27 kernel.
 */
# ifndef min
#  define min(a,b) (((a)<(b))?(a):(b))
# endif
# ifndef max
#  define max(a,b) (((a)>(b))?(a):(b))
# endif

/*
 * Yes, this is ugly, but its needed because Red Hat added the
 * "atomic read for tty count" patch to their 2.4 kernels, but
 * kernel.org (and other vendors) did not.
 */
# if defined(REDHAT_AS_30)  || defined(REDHAT_ES_30) || defined(REDHAT_WS_30) || \
    defined(REDHAT_AS_3)   || defined(REDHAT_ES_3)  || defined(REDHAT_WS_3)  || \
    defined(REDHATAS30)    || defined(REDHATES30)   || defined(REDHATWS30)   || \
    defined(REDHATAS3)     || defined(REDHATES3)    || defined(REDHATWS3)    || \
    defined(REDHAT_FEDORA) || defined(FEDORA)

#  define DGAP_GET_TTY_COUNT(x) (atomic_read(&(x->count)))

# else

#  define DGAP_GET_TTY_COUNT(x) (x->count)

# endif

/* Linux 2.4.29+ all have the new Alan Cox tty locking in it... */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,29)
#  define NEW_TTY_LOCKING
# endif

# ifndef NEW_TTY_LOCKING

#  define tty_ldisc_flush(tty) \
	if ((tty)->ldisc.flush_buffer) { \
		(tty)->ldisc.flush_buffer((tty)); \
	}

#  define tty_wakeup(tty) \
	if ((test_bit(TTY_DO_WRITE_WAKEUP, &(tty)->flags)) && \
	    (tty)->ldisc.write_wakeup) { \
		((tty)->ldisc.write_wakeup)((tty)); \
	}

#  define tty_ldisc_ref(tty) (&(tty)->ldisc)

#  define tty_ldisc_deref(ld) do {} while (0)

# endif

# define DGAP_MUTEX_INITIALIZER(name)	__MUTEX_INITIALIZER(name)

# define DGAP_PCI_REGISTER_DRIVER(driver) pci_module_init(driver)




# else



# error "this driver does not support anything below the 2.4 kernel series."



# endif

#endif /* ! __DGAP_KCOMPAT_H */
