#!/bin/sh
#set -x
# usertest.sh
# Allow user to set a few typical "utest" options, then run the test

# Point to our empty dialogrc file
DIALOGRC=/etc/dgap/dialogrc
export DIALOGRC

TMP=/tmp/$$.tmp
TMPIN=/tmp/$$.tmpin

# Digi DGAP devices
D_DEVLIST=/tmp/$$.ddevlist
D_DEVLISTNAME=/tmp/$$.ddevlistname
DEVLISTNAME=

# Other devices (e.g: built-in COM ports)
O_DEVLIST=/tmp/$$.odevlist

# Default options to utest:
#     -l1000      1000 bytes of random data
#     -b 57600    57600 baud
#     -A          Allow any keypress to abort tests in progress
#     -R          Display summary report at end of tests
#
# Optional settings:
#     -B          Bidirectional (run each test in both directions)
#     -L          Loopback
#
# N.B. Port names are presented to the user as Port1, Port2,...
# so the device nodes must be created using these names too.  That
# is: instead of (or in addition to)  /dev/ttyCP0, /dev/ttyCP1... we
# need /dev/Port1, /dev/Port2...  Both the names and the indices are
# different since usertest indices start at "1".
#

UTEST_OPTS_DEF="-l1000 -b 57600 -A -R"
VERSION="1.5"
VERMES="Digi DGAP Diagnostics Utility   Version "${VERSION}

SELECTMES1="Use arrow keys to select"
SELECTMES2="then press Enter"
#SELECTMES="Use arrow keys to select an option, then press Enter"
SELECTMES=${SELECTMES1}" an option, "${SELECTMES2}
SELECTINMES=${SELECTMES1}" Second Port, "${SELECTMES2}
SELECTOUTMES=${SELECTMES1}" First Port, "${SELECTMES2}
PROCFILE=/proc/dgap/*/info

# Default board is 0
BOARD=0

# Number of boards is 0
TOTAL_BOARDS=0

# Number of ports is 0
NPORTS=0

DRIVER_VERSION=



dohelp()
{
	helpfile_basename="dgaptest_help.txt"
	helpfile=${helpfile_basename}
	if [ -f ${helpfile_basename} ]			# Local directory?
	then
		helpfile=${helpfile_basename}
	elif [ -f /etc/dgap/${helpfile_basename} ]	# /etc/dgap?
	then
		helpfile="/etc/dgap/"${helpfile_basename}
	elif [ -f /etc/${helpfile_basename} ]	# /etc?
	then
		helpfile="/etc/"${helpfile_basename}
	else
		/etc/dgap/dialog \
		--backtitle "${VERMES}" \
		--title "Description of Available Tests" \
		--msgbox \
		"Cannot find help file: dgaptest_help.txt" \
		18 75
		return
	fi

	/etc/dgap/dialog \
	--backtitle "${VERMES}" \
	--title "Description of Available Tests" \
	--textbox ${helpfile} 18 75
}

get_board_name()
{
	while read a b c rest
	do
		printf "$rest" >/tmp/dgap.tmp2
		return
	done
}

get_board_irq()
{
	while read a b irq rest
	do
		printf "$irq" >/tmp/dgap.tmp2
		return
	done
}

get_board_bus()
{
	while read a b bus rest
	do
		printf "$bus" >/tmp/dgap.tmp2
		return
	done
}

get_board_slot()
{
	while read a b slot rest
	do
		printf "$slot" >/tmp/dgap.tmp2
		return
	done
}

get_board_ports()
{
	while read a b c d ports rest
	do
		printf "$ports" >/tmp/dgap.tmp2
		return
	done
}

showproc()
{
	tmpi=0
	rmfile /tmp/dgap.out

	printf "Found $TOTAL_BOARDS adapter(s):\n\n" >> /tmp/dgap.out

	while [ $tmpi -lt $TOTAL_BOARDS ]
	do
		grep "Board Name" /proc/dgap/$tmpi/info > /tmp/dgap.tmp
		get_board_name < /tmp/dgap.tmp
		rmfile /tmp.dgap.tmp
		name=`cat /tmp/dgap.tmp2`
		rmfile /tmp.dgap.tmp2

		grep "Interrupt" /proc/dgap/$tmpi/info > /tmp/dgap.tmp
		get_board_irq < /tmp/dgap.tmp
		rmfile /tmp.dgap.tmp
		irq=`cat /tmp/dgap.tmp2`
		rmfile /tmp.dgap.tmp2

		grep "Bus" /proc/dgap/$tmpi/info > /tmp/dgap.tmp
		get_board_bus < /tmp/dgap.tmp
		rmfile /tmp.dgap.tmp
		bus=`cat /tmp/dgap.tmp2`
		rmfile /tmp.dgap.tmp2

		grep "Slot" /proc/dgap/$tmpi/info > /tmp/dgap.tmp
		get_board_slot < /tmp/dgap.tmp
		rmfile /tmp.dgap.tmp
		slot=`cat /tmp/dgap.tmp2`
		rmfile /tmp.dgap.tmp2

		tmpi=`expr $tmpi + 1`
		printf "  $tmpi. %-25sIRQ: ${irq}\tPCI(Bus:$bus,Slot:$slot).\n" "$name" >> /tmp/dgap.out
	done

	cp /tmp/dgap.out ${TMP}

	# Make sure it's writable so we can overwrite or remove it without error
	chmod +w ${TMP}
	/etc/dgap/dialog \
	--backtitle "${VERMES}" \
	--title "List of Digi DGAP Adapters in this Machine" \
	--textbox ${TMP} 18 75

}

# Map non-native device index (begin counting at 1) into a
# Linux device name (begin counting at 0).  Also, if argument
# is a letter, treat it as an index into COM1, COM2... (begin
# counting with "A")
map_port_for_diags()
{
	
	if [ $1 = "A" ]
	then
		mapped_port=/dev/ttyS0
	elif [ $1 = "B" ]
	then
		mapped_port=/dev/ttyS1
	else
		mapped_port=/dev/Port$1
	fi
}

# Map non-native device index (begin counting at 1) into a
# Linux device name (begin counting at 0).  Also, if argument
# is a letter, treat it as an index into COM1, COM2... (begin
# counting with "A")
map_port_for_driver()
{
	port=$1

	if [ $1 = "A" ]
	then
		mapped_port=/dev/ttyS0
	elif [ $1 = "B" ]
	then
		mapped_port=/dev/ttyS1
	else
		for word in ${DEVLISTNAME}
		do
			port=`expr "$port" - "1"`

			if [ "$port" -le "0" ]
			then
				mapped_port=$word
				break
			fi
		done
	fi
}

rmfile()
{
	if [ -f $1 ]
	then
		#echo "In rmfile, wanna rm "$1
		rm -f $1
	fi
}

welcome()
{
	local ret

	# WELCOME SCREEN
	/etc/dgap/dialog \
	--backtitle "${VERMES}" \
	--title "Main Menu" --menu "${SELECTMES}" 18 70 12 \
	"Run" "Run Standard Tests" \
	"Help" "Describe Tests" \
	"Info"	"List Digi Adapters in this Machine" \
	"Exit" "Exit" 2>${TMP}

	if [ $? -ne 0 ]
	then
		#ret="Cancel"
		ret="Exit"
	else
		ret=`tail -1 ${TMP}`
	fi

	# Cancel button on this screen leaves program
	if [ ret = "Cancel" ]
	then
		ret="Exit"
	fi

	# action is global
	action=$ret
}

select_testtype()
{
	local ret

	# WELCOME SCREEN
	/etc/dgap/dialog \
	--backtitle "${VERMES}" \
	--title "Select Test Type" \
	--menu "${SELECTMES}" 18 75 12 \
	"External" "External loopback (one port; need loopback plug)" \
	"Port-Port" "Port to port test (need null modem cable)" 2>${TMP}

	# testtype is global
	if [ $? -ne 0 ]
	then
		testtype="Cancel"
	else
		testtype=`tail -1 ${TMP}`
	fi

}

select_output()
{
	# Internal loopback tests only work on Digi Neo ports: show only these
	if [ $testtype = "Internal" ]
	then
		DLIST="${D_DEVLISTNAME}"
	else
		DLIST="${D_DEVLISTNAME} ${O_DEVLIST}"
	fi

	# SELECT OUTPUT PORT
	echo -n "/etc/dgap/dialog \
	--title \"Select First Port\" \
	--backtitle \"${VERMES}\" \
	--menu \"${SELECTOUTMES}\" 18 70 12 "> ${TMPIN}
	#for i in `cat ${D_DEVLIST} ${O_DEVLIST}`
	for i in `cat ${DLIST}`
	do
	#echo -n " "$i" \" \"" >> ${TMPIN}
	echo -n " "$i >> ${TMPIN}
	done
	. ${TMPIN} 2>${TMP}

	# action and output are globals
	if [ $? -ne 0 ]
	then
		action="Cancel"
	else
		action="Continue"
	fi
	port=`tail -1 ${TMP}`

	# Map port name to Linux native name
	if [ "$action" = "Continue" ]
	then
		map_port_for_driver $port
		outport=${mapped_port}
	fi

}

select_input()
{
	# SELECT INPUT PORT
	echo -n "/etc/dgap/dialog \
	--title \"Select Second Port\" \
	--backtitle \"${VERMES}\" \
	--menu \"${SELECTINMES}\" 18 70 12 "> ${TMPIN}
	for i in `cat ${D_DEVLISTNAME} ${O_DEVLIST}`
	do
	echo -n " "$i >> ${TMPIN}
	done
	. ${TMPIN} 2>${TMP}

	# action and output are globals
	if [ $? -ne 0 ]
	then
		action="Cancel"
	else
		action="Continue"
	fi

	port=`tail -1 ${TMP}`

	# Map port name to Linux native name
	if [ "$action" = "Continue" ]
	then
		map_port_for_driver $port
		inport=${mapped_port}
	fi
}

cleanup()
{
	rmfile ${TMP}
	rmfile ${TMPIN}
	rmfile ${D_DEVLIST}
	rmfile ${O_DEVLIST}
	rmfile ${D_DEVLISTNAME}
	rm -fr /tmp/dgap*
}


get_number_of_boards()
{
	while read a b boards rest
	do
		printf $boards
		return
	done
}


get_driver_version()
{
	while read a version rest
	do
		printf $version
		return
	done
}


parse_mknod_output()
{
	while read name major minor count start rest
	do
		tmpj=0
		while [ $tmpj -lt $count ]
		do
			offset=`expr $start + $tmpj`

			# Pad out port name
			case $offset in
 				?)
					port=0${offset}
				;;
				??)
					port=${offset}
				;;
				???)
					port=${offset}
				;;
			esac
  
			DEVLISTNAME="${DEVLISTNAME} /dev/${name}${port}"
			n=`expr ${NPORTS} + $tmpj + 1`
			echo "$n /dev/${name}${port}" >> ${D_DEVLISTNAME}
			tmpj=`expr $tmpj + 1`
		done

		NPORTS=`expr $NPORTS + $count`

	done
}



get_number_of_ports()
{
	rmfile /tmp/dgap.out
	rmfile /tmp/dgap.mknod


	tmpi=0
	while [ $tmpi -lt $TOTAL_BOARDS ]
	do
		grep "Ports" /proc/dgap/$tmpi/info > /tmp/dgap.tmp
		get_board_ports < /tmp/dgap.tmp
		rmfile /tmp.dgap.tmp
		ports=`cat /tmp/dgap.tmp2`
		rmfile /tmp.dgap.tmp2

		cat /proc/dgap/${tmpi}/mknod | grep tty > /tmp/dgap.mknod
		parse_mknod_output < /tmp/dgap.mknod

		tmpi=`expr $tmpi + 1`
	done
}


init()
{
	# Find number of boards installed...
	if [ -f /proc/dgap/info ]
	then
		grep "Total Boards:" /proc/dgap/info >/tmp/dgap.tmp
		TOTAL_BOARDS=`get_number_of_boards < /tmp/dgap.tmp`
		rmfile /tmp/dgap.tmp

		grep "Driver:" /proc/dgap/info >/tmp/dgap.tmp
		DRIVER_VERSION=`get_driver_version < /tmp/dgap.tmp`
		rmfile /tmp/dgap.tmp
	fi

	# How many ports did the dgap driver find?
	get_number_of_ports

	if [ -f ${D_DEVLIST} ]
	then
		rmfile ${D_DEVLIST}
	fi
	if [ -f ${O_DEVLIST} ]
	then
		rmfile ${O_DEVLIST}
	fi

	i=1	# For user purposes, port counts start at 1
	while [ $i -le ${NPORTS} ]
	do
		echo ${i} Port${i} >>${D_DEVLIST}
		i=`expr $i + 1`
	done
	# Add built-in serial ports (COM1...) to the list
	#ls /dev/ttyS* >>${O_DEVLIST} 2>/dev/null
	echo A COM1 >>${O_DEVLIST} 2>/dev/null
	echo B COM2 >>${O_DEVLIST} 2>/dev/null
}

select_numtests()
{
	# NUMBER OF TESTS TO RUN
	/etc/dgap/dialog \
	--backtitle "Digi DGAP Diagnostics Utility   Version "$VERSION \
	--title "Select number of tests to run:" \
	--menu "${SELECTMES}" 18 70 12 \
	"Single Test" " " "10 Tests" " " "Continuous Tests" " " 2>${TMP}

	# action and numtests  are globals
	if [ $? -ne 0 ]
	then
		action="Cancel"
	else
		action="Continue"
	fi
	numtests=`tail -1 ${TMP}`
}

	



init

showproc

QUIT=0

while [ $QUIT -ne 1 ]
do
	# Set test defaults
	UTEST_OPTS=${UTEST_OPTS_DEF}

	welcome
	case $action in
		"Help" ) dohelp; continue ;;
		"Cancel" ) continue ;;
		"Info" ) showproc; continue ;;
		"Exit" ) QUIT=1; continue ;;
	esac


	if [ "$TOTAL_BOARDS" -eq "0" ]
	then
		/etc/dgap/dialog --msgbox "No DGAP boards found, cannot run Tests." 5 60 $cols 2>/dev/null
		continue;
	fi

		
	select_testtype
	case $testtype in
		"Internal" ) UTEST_OPTS=${UTEST_OPTS_DEF}" -L" ;;
		"Cancel" ) continue;;
	esac

	# set "outport"
	select_output
	if [ ${action} = "Cancel" ]
	then
		continue;
	fi

	# Do we need a second port specification?
	if [ ${testtype} = "Port-Port" ]
	then
		UTEST_OPTS=${UTEST_OPTS}" -B"	# Add bi-directional
		# set "inport"
		select_input
		if [ ${action} = "Cancel" ]
		then
			continue;
		fi
	else
		inport=$outport
	fi
	
	select_numtests
	if [ ${action} = "Cancel" ]
	then
		continue;
	fi

	if [ "${numtests}" = "Continuous Tests" ]
	then
		/etc/dgap/utest -n-1 ${UTEST_OPTS} -i ${inport} -o ${outport}
	elif [ "${numtests}" = "10 Tests" ]
	then
		/etc/dgap/utest -n10 ${UTEST_OPTS} -i ${inport} -o ${outport}
	else
		/etc/dgap/utest ${UTEST_OPTS} -i ${inport} -o ${outport}
	fi

	echo "Press Enter to continue..."
	read X

done

cleanup
