# imm.sh
# Run FCC "immunity" tests -- run bidirectional tests at 920K baud
# between 4 pairs of ports.  The last 50 errors (if any) are collected
# in /tmp/imm_err[1234].log.
#
# Peter Berger
# History:
# Version 1.0: PEB: January 16 2001
# initial version
#
# utest output will go to named pipes (FIFOs) /tmp/imm_fifo[123...]
FIFO=/tmp/imm_fifo
# utest errors will go to named pipes (FIFOs) /tmp/imm_fifo_err[123...]
FIFO_ERR=/tmp/imm_fifo_err
# the most recent error lines will be in normal files /tmp/imm_error[123...].log
ERR=/tmp/imm_error

# Run directory: explicitly look for files in this directory.  Leave
# undefined to use copies in your current PATH
#RDIR=./

for i in 1 2 3 4
do
	if [ -f ${FIFO}$i -o -p ${FIFO}$i ]
	then
		rm ${FIFO}$i
	fi
	mkfifo ${FIFO}$i

	if [ -f ${FIFO_ERR}$i -o -p ${FIFO_ERR}$i ]
	then
		rm ${FIFO_ERR}$i
	fi
	mkfifo ${FIFO_ERR}$i
done



# Setup test A
${RDIR}svfifo -n 50 -i ${FIFO_ERR}1 -o ${ERR}1.log&
PID1_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP0 -o /dev/ttyCP1 -b921600 \
>${FIFO}1 2>${FIFO_ERR}1&
PID1=$!


# Setup test B
${RDIR}svfifo -n 50 -i ${FIFO_ERR}2 -o ${ERR}2.log&
PID2_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP2 -o /dev/ttyCP3 -b921600 \
>${FIFO}2 2>${FIFO_ERR}2&
PID2=$!


# Setup test C
${RDIR}svfifo -n 50 -i ${FIFO_ERR}3 -o ${ERR}3.log&
PID3_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP4 -o /dev/ttyCP5 -b921600 \
>${FIFO}3 2>${FIFO_ERR}3&
PID3=$!

# Setup test D
${RDIR}svfifo -n 50 -i ${FIFO_ERR}4 -o ${ERR}4.log&
PID4_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP6 -o /dev/ttyCP7 -b921600 \
>${FIFO}4 2>${FIFO_ERR}4&
PID4=$!

# Call the the display program
${RDIR}disp \
-f ${FIFO}1 -x0 -y0 -t5 -T"Test A " \
-f ${FIFO}2 -x0 -y6  -t5 -T"Test B " \
-f ${FIFO}3 -x0 -y12 -t5 -T"Test C " \
-f ${FIFO}4 -x0 -y18 -t5 -T"Test D "

# kill the background processes
#kill -s 15 \
#	${PID1_ERR} ${PID1} \
#	${PID2_ERR} ${PID2} \
#	${PID3_ERR} ${PID3} \
#	${PID4_ERR} ${PID4}

