# imm2.sh
# Run FCC "immunity" tests -- run bidirectional tests at 920K baud
# on all 8 Classicboard port, assuming all ports are terminated with
# external loopback plugs.
# The last 50 errors (if any) are collected
# in /tmp/imm_err[1234].log.
#
# Peter Berger
# History:
# Version 1.0: PEB: January 16 2001
# initial version
#
# utest output will go to named pipes (FIFOs) /tmp/imm_fifo[123...]
FIFO=/tmp/imm_fifo
# utest errors will go to named pipes (FIFOs) /tmp/imm_fifo_err[123...]
FIFO_ERR=/tmp/imm_fifo_err
# the most recent error lines will be in normal files /tmp/imm_error[123...].log
ERR=/tmp/imm_error

# Run directory: explicitly look for files in this directory.  Leave
# undefined to use copies in your current PATH
#RDIR=./

for i in 1 2 3 4 5 6 7 8
do
	if [ -f ${FIFO}$i -o -p ${FIFO}$i ]
	then
		rm ${FIFO}$i
	fi
	mkfifo ${FIFO}$i

	if [ -f ${FIFO_ERR}$i -o -p ${FIFO_ERR}$i ]
	then
		rm ${FIFO_ERR}$i
	fi
	mkfifo ${FIFO_ERR}$i
done



# Setup test A
${RDIR}svfifo -n 50 -i ${FIFO_ERR}1 -o ${ERR}1.log&
PID1_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP0 -o /dev/ttyCP0 -b921600 \
>${FIFO}1 2>${FIFO_ERR}1&
PID1=$!


# Setup test B
${RDIR}svfifo -n 50 -i ${FIFO_ERR}2 -o ${ERR}2.log&
PID2_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP1 -o /dev/ttyCP1 -b921600 \
>${FIFO}2 2>${FIFO_ERR}2&
PID2=$!


# Setup test C
${RDIR}svfifo -n 50 -i ${FIFO_ERR}3 -o ${ERR}3.log&
PID3_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP2 -o /dev/ttyCP2 -b921600 \
>${FIFO}3 2>${FIFO_ERR}3&
PID3=$!

# Setup test D
${RDIR}svfifo -n 50 -i ${FIFO_ERR}4 -o ${ERR}4.log&
PID4_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP3 -o /dev/ttyCP3 -b921600 \
>${FIFO}4 2>${FIFO_ERR}4&
PID4=$!

# Setup test E
${RDIR}svfifo -n 50 -i ${FIFO_ERR}5 -o ${ERR}5.log&
PID5_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP4 -o /dev/ttyCP4 -b921600 \
>${FIFO}5 2>${FIFO_ERR}5&
PID5=$!

# Setup test F
${RDIR}svfifo -n 50 -i ${FIFO_ERR}6 -o ${ERR}6.log&
PID6_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP5 -o /dev/ttyCP5 -b921600 \
>${FIFO}6 2>${FIFO_ERR}6&
PID6=$!

# Setup test G
${RDIR}svfifo -n 50 -i ${FIFO_ERR}7 -o ${ERR}7.log&
PID7_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP6 -o /dev/ttyCP6 -b921600 \
>${FIFO}7 2>${FIFO_ERR}7&
PID7=$!

# Setup test H
${RDIR}svfifo -n 50 -i ${FIFO_ERR}8 -o ${ERR}8.log&
PID8_ERR=$!
${RDIR}utest -B -P0x01020304050607080910 -n-1 -i /dev/ttyCP7 -o /dev/ttyCP7 -b921600 \
>${FIFO}8 2>${FIFO_ERR}8&
PID8=$!

# Call the the display program
${RDIR}disp \
-f ${FIFO}1 -x0 -y0  -t3 -T"Test A " \
-f ${FIFO}2 -x0 -y3  -t3 -T"Test B " \
-f ${FIFO}3 -x0 -y6  -t3 -T"Test C " \
-f ${FIFO}4 -x0 -y9  -t3 -T"Test D " \
-f ${FIFO}5 -x0 -y12 -t3 -T"Test E " \
-f ${FIFO}6 -x0 -y15 -t3 -T"Test F " \
-f ${FIFO}7 -x0 -y18 -t3 -T"Test G " \
-f ${FIFO}8 -x0 -y21 -t3 -T"Test H "

# kill the background processes
#kill -s 15 \
#	${PID1_ERR} ${PID1} \
#	${PID2_ERR} ${PID2} \
#	${PID3_ERR} ${PID3} \
#	${PID4_ERR} ${PID4}

