# run.sh
# Run some sample "utest" tests, display their outputs on
# a single windowed console, and collect the last N lines
# errors (if any) in separate error log files.
#
# Peter Berger
# History:
# Version 1.0: PEB: January 14 2001
# initial version
#
# Note: svfifo must be run before the corresponding utest since
# bash creates its stderr output files (including the FIFOs
# we use to collect this info) in blocking mode and will hang
# unless someone is already listening on the FIFO.

# utest output will go to named pipes (FIFOs) /tmp/fifo[123...]
FIFO=/tmp/fifo
# utest errors will go to named pipes (FIFOs) /tmp/fifo_err[123...]
FIFO_ERR=/tmp/fifo_err
# the most recent error lines will be in normal files /tmp/error[123...].log
ERR=/tmp/error

# Run directory: explicitly look for files in this directory.  Leave
# undefined to use copies in your current PATH
#RDIR=./

for i in 1 2 3 4
do
	if [ -f ${FIFO}$i -o -p ${FIFO}$i ]
	then
		rm ${FIFO}$i
	fi
	mkfifo ${FIFO}$i

	if [ -f ${FIFO_ERR}$i -o -p ${FIFO_ERR}$i ]
	then
		rm ${FIFO_ERR}$i
	fi
	mkfifo ${FIFO_ERR}$i
done



# Setup test 1
${RDIR}/svfifo -n 50 -i ${FIFO_ERR}1 -o ${ERR}1.log&
PID1_ERR=$!
${RDIR}/utest -P0x01020304050607080910 -n-1 -i /dev/ttyCP0 -o /dev/ttyCP1 -b921600 \
>${FIFO}1 2>{FIFO_ERR}1&
PID1=$!


# Setup test 2
${RDIR}/svfifo -n 50 -i ${FIFO_ERR}2 -o ${ERR}2.log&
PID2_ERR=$!
${RDIR}/utest -l100 -n-1 -o /dev/ttyCP2 -b921600 >${FIFO}2 2>${FIFO_ERR}2&
PID2=$!


# Setup test 3
${RDIR}/svfifo -n 50 -i ${FIFO_ERR}3 -o ${ERR}3.log&
PID3_ERR=$!
${RDIR}/utest -l100 -n-1 -i /dev/ttyCP3 -o /dev/ttyCP4 -b230400 \
>${FIFO}3 2>${FIFO_ERR}3&
PID3=$!

# Setup test 4
${RDIR}/svfifo -n 50 -i ${FIFO_ERR}4 -o ${ERR}4.log&
PID4_ERR=$!
${RDIR}/utest -l100 -n-1 -o /dev/ttyCP5 -b921600 >${FIFO}4 2>${FIFO_ERR}4&
PID4=$!


# Call the the display program
${RDIR}/disp \
-f ${FIFO}1 -x0 -y0 -t5 -T"Test A " \
-f ${FIFO}2 -x0 -y6  -t5 -T"Test B " \
-f ${FIFO}3 -x0 -y12 -t5 -T"Test C " \
-f ${FIFO}4 -x0 -y18 -t5 -T"Test D "

# kill the background processes
for i in 1 2 3 4
do
kill $PID$i $PID$i_ERR
done
